import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;


public class scribble_mdi
{
	static class ScribblePanel extends JPanel
	{
		private static final long serialVersionUID = 1L;
		
		private BufferedImage bild = null;
		private Point lastPosition;
		private Point textPosition;
		
		private int imageNumber = 1;
		
		public void saveImage()
		{
			try 
			{
				File datei = new File(String.format("bild_%d.png", imageNumber));
				ImageIO.write(bild, "png", datei);
			}
			catch (IOException e) 
			{
				JOptionPane.showMessageDialog(this, e.getMessage());
			}
		}
		
		public void drawString(String key)
		{
			Graphics g = bild.getGraphics();
			g.setColor(new Color(255,0,0));
			g.drawString(key, textPosition.x, textPosition.y);
			g.dispose();
			
			Rectangle toDraw = new Rectangle(textPosition, new Dimension(20,20));
			toDraw.grow(20, 20);
			ScribblePanel.this.paintImmediately(toDraw);
			
			if(textPosition.x > ScribblePanel.this.getWidth())
			{
				textPosition.x = 20;
				textPosition.y += 20;
			}
			else textPosition.x += 20*key.length();
		}
		
		
		public void clearImage()
		{
			textPosition = new Point(30,20);
			Graphics g = bild.getGraphics();
			g.setColor(new Color(0,0,50));
			g.fillRect(0, 0, bild.getWidth(), bild.getHeight());
			g.dispose();
			repaint();
		}
		
		public ScribblePanel(int imageNumber)
		{
			this.imageNumber = imageNumber;
			bild = new BufferedImage(100, 100,  BufferedImage.TYPE_INT_ARGB);
			clearImage();
			
			this.addComponentListener(new ComponentAdapter()
			{
				@Override
				public void componentResized(ComponentEvent e)
				{
					if(ScribblePanel.this.getWidth() == 0 || ScribblePanel.this.getHeight() == 0) return;
					
					BufferedImage neuesBild = new BufferedImage(ScribblePanel.this.getWidth(), ScribblePanel.this.getHeight(),  BufferedImage.TYPE_INT_ARGB);
					Graphics g = neuesBild.getGraphics();
					// Hintergrund
					g.setColor(new Color(0,0,50));
					g.fillRect(0, 0, neuesBild.getWidth(), neuesBild.getHeight());
					
					// Bildinhalt kopieren:
					g.drawImage(bild, 0, 0, null);
					// Graphicsobjekt loeschen:
					g.dispose();
					bild = neuesBild;
				}
			});
			
			
			this.addMouseMotionListener(new MouseMotionListener()
			{

				@Override
				public void mouseDragged(MouseEvent e) 
				{
					if((e.getModifiersEx() & InputEvent.BUTTON1_DOWN_MASK) != 0)
					{
						if(lastPosition != null)
						{
							Graphics g = bild.getGraphics();
							g.setColor(new Color(255,255,0));
							g.drawLine(lastPosition.x, lastPosition.y, e.getX(), e.getY());
							g.dispose();
							
							// optimieren, damit es fluessig laeuft:
							Rectangle toDraw = new Rectangle(lastPosition);
							toDraw.add(e.getPoint());
							toDraw.grow(5, 5);
							ScribblePanel.this.paintImmediately(toDraw);
							
						}
						lastPosition = new Point(e.getPoint());
						
					}
					
				}

				@Override
				public void mouseMoved(MouseEvent e) {	
					if(lastPosition != null) lastPosition = null;
				}
				
			});
			
			this.addKeyListener(new KeyListener() {

				@Override
				public void keyPressed(KeyEvent e) 
				{
					
					if(e.getKeyCode() == KeyEvent.VK_C && e.isControlDown())
						clearImage();
					else if(e.getKeyCode() == KeyEvent.VK_ENTER)
					{
						textPosition.x = 20;
						textPosition.y += 20;
					}
					else if(e.getKeyCode() == KeyEvent.VK_BACK_SPACE)
					{
						textPosition.x -= 20;
					}
				}

				@Override
				public void keyReleased(KeyEvent e) {
					
				}

				@Override
				public void keyTyped(KeyEvent e) 
				{
					if(Character.isISOControl(e.getKeyChar())) return;
					drawString("" + e.getKeyChar());
					
				}
				
			});
			
		}
		
		@Override
		public void paintComponent(Graphics g)
		{
			if(bild == null) return;
			g.drawImage(bild, 0, 0, null);
		}

		@Override
		public boolean isFocusable()
		{
			return true;
		}
		
		

	}
	
	static class MyFrame extends JFrame
	{
		private static final long serialVersionUID = 1L;
		private int imageNumber = 0;
		JDesktopPane desktoppane = new JDesktopPane();
		HashMap<JInternalFrame, ScribblePanel> scribbles = new HashMap<JInternalFrame, ScribblePanel>();
		
		MyFrame()
		{
			super("Scribble MDI");
			
			@SuppressWarnings("serial")
			class BeendenAction extends AbstractAction
			{
				BeendenAction()
				{
					putValue(Action.NAME, "Beenden");
					putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK)); // Ctrl+q
				}
				@Override
				public void actionPerformed(ActionEvent e) 
				{
					System.exit(0);
				}
			}
			@SuppressWarnings("serial")
			class NeuAction extends AbstractAction
			{
				NeuAction()
				{
					putValue(Action.NAME, "Neu");
					putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK)); // Ctrl+q
				}
				@Override
				public void actionPerformed(ActionEvent e) 
				{
					ScribblePanel scribble = new ScribblePanel(++imageNumber);
					JInternalFrame scribbleFrame = new JInternalFrame(String.format("Scribble #%d", imageNumber), true, true, true, true);
					scribbleFrame.add(scribble);
					
					desktoppane.add(scribbleFrame);
					scribbleFrame.setBounds(25, 25, 500, 500);
					scribbleFrame.setVisible(true);
					
					scribbles.put(scribbleFrame, scribble);
				}
			}
			
			@SuppressWarnings("serial")
			class SaveAction extends AbstractAction
			{
				SaveAction()
				{
					putValue(Action.NAME, "Speichern");
					putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK)); // Ctrl+s
				}
				@Override
				public void actionPerformed(ActionEvent e) 
				{
					JInternalFrame internal = desktoppane.getSelectedFrame();
					if(internal == null) return;
					scribbles.get(internal).saveImage();
				}
			}
			
			
			SaveAction saveaction = new SaveAction();
			BeendenAction beendenaction = new BeendenAction();
			NeuAction neuaction = new NeuAction();

			JMenuBar menubar = new JMenuBar();
			JMenu datei = new JMenu("Datei");
			menubar.add(datei);
			
			JMenuItem neu = new JMenuItem();
			neu.setAction(neuaction);
			datei.add(neu);
			
			JMenuItem save = new JMenuItem();
			save.setAction(saveaction);
			datei.add(save);
			
			datei.add(new JSeparator());
			
			JMenuItem quit = new JMenuItem();
			quit.setAction(beendenaction);
			datei.add(quit);
			

			
			
			this.setJMenuBar(menubar);
			
			neuaction.actionPerformed(null);
			
			Container c = super.getContentPane(); 
			c.add(desktoppane);
			super.pack();
		}
		
	}
	
	public static void main(String[] a)
	{
		JFrame.setDefaultLookAndFeelDecorated(true); 
		SwingUtilities.invokeLater(new Runnable()
		{
			@Override
			public void run() 
			{
				MyFrame f = new MyFrame();
				f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				f.setVisible(true);
			}			
		}
				
		);
	}
	
	
}